#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "global.h"
#include "RFM01.h"

void RFM01_send_cmd(unsigned int command)
  {
	unsigned char i;
	RFM01_PORT &= ~(1<<RFM01_CS);				// CS auf L

	for (i=0; i<16; i++)
		{
			if (command & 0x8000)
			  {
			  	RFM01_PORT |= (1<<RFM01_SDI);		// DATA auf H
			  }
			else
			  {
			  	RFM01_PORT &= ~(1<<RFM01_SDI);		// DATA auf L
			  }
			asm("nop");
		  	RFM01_PORT |= (1<<RFM01_SCK);			// CLK auf H
			asm("nop");
			asm("nop");
		  	RFM01_PORT &= ~(1<<RFM01_SCK);			// CLK auf L
			command = (command << 1);				// nchstes Bit nach oben
		}
	RFM01_PORT |= (1<<RFM01_CS);					// CS auf H
  }

void RFM01_init(void)
	{
		RFM01_PORT = (1<<RFM01_CS) | (1<<RFM01_FIFO);			//	 Ausgang H	
		RFM01_DDR  = (1<<RFM01_SDI) | (1<<RFM01_SCK) | (1<<RFM01_CS) | (1<<RFM01_FIFO);	// Ausgnge

		RFM01_send_cmd(0x0000);						// Status read

		RFM01_send_cmd(0x8958);						

		RFM01_send_cmd(0xA620);					

		RFM01_send_cmd(0xC810);						

		RFM01_send_cmd(0xC69B);					 

		RFM01_send_cmd(0xC42A);					

		RFM01_send_cmd(0xC280);					 

		RFM01_send_cmd(0xCE88);					

		RFM01_send_cmd(0xCE8B);						

		RFM01_send_cmd(0xC081);						// Bandbreite
	}

void RFM01_read_fifo(char *data_buf, char len)
	{

		unsigned char byte = 0, i, j;

		RFM01_send_cmd(0xC081);						// Empfnger ein

		RFM01_send_cmd(0xCE85);						// set FIFO mode

		RFM01_send_cmd(0xCE87);						// enable FIFO

  	RFM01_PORT &= ~(1<<RFM01_SCK);				// CLK auf L
  	RFM01_PORT &= ~(1<<RFM01_SDI);				// DATA auf L

		for (i=0; i<len; i++)
		  {
		  	RFM01_PORT &= ~(1<<RFM01_CS);			// CS auf L
				asm("nop");
				asm("nop");
			while (!(RFM01_PIN & (1<<RFM01_SDO)))	// warten bis SDO 1 -> FFIL 
			  {
			  }
		  					
			RFM01_PORT &= ~(1<<RFM01_FIFO);							// Read FIFO ein (nFFS auf Low)

			byte = 0;

			for (j=0; j<8; j++)
			  { 		
				byte = (byte << 1);					// eins hher schieben
				if ((RFM01_PIN & (1<<RFM01_SDO)) == (1<<RFM01_SDO))	// Bit 1?
			  	  {
			  			byte = (byte | 0x01);				// ja
			  	  }
				asm("nop");
			  	RFM01_PORT |= (1<<RFM01_SCK);		// CLK auf H
				asm("nop");
				asm("nop");
			  	RFM01_PORT &= ~(1<<RFM01_SCK);		// CLK auf L
			  }
			*data_buf++ = byte;
			RFM01_PORT |= (1<<RFM01_CS);			// CS auf H
			RFM01_PORT |= (1<<RFM01_FIFO);		// nFFS auf H
		}

	  RFM01_send_cmd(0xC080);						// Empfnger aus

	}

unsigned int RFM01_read_status(void)
  {
		unsigned int status = 0;
		unsigned char i = 0;

		RFM01_PORT &= ~(1<<RFM01_SCK);				// CLK auf L
 		RFM01_PORT &= ~(1<<RFM01_SDI);				// DATA auf L

		RFM01_PORT &= ~(1<<RFM01_CS);					// CS auf L

		for (i=0; i<16; i++)
			{
				status = (status << 1);						// eins hher schieben
				if ((RFM01_PIN & (1<<RFM01_SDO)) == (1<<RFM01_SDO))	// Bit 1?
			    {
			  		status = (status | 0x0001);		// ja
			    }
				asm("nop");
			  	RFM01_PORT |= (1<<RFM01_SCK);		// CLK auf H
				asm("nop");
				asm("nop");
				asm("nop");
				asm("nop");
			  RFM01_PORT &= ~(1<<RFM01_SCK);		// CLK auf L
			}
		RFM01_PORT |= (1<<RFM01_CS);					// CS auf H

		return status;
	}



