
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "global.h"
#include "UART.h"
#include "RFM01.h"
#include "Tabellen.h"

char check_paket(char *, char);
void show_sensor(char *);
void show_temp(char *, uint8_t);
void show_feuchte(char *, uint8_t);
void show_druck(char *, uint8_t);
void show_switch(char *, uint8_t);
void show_status(uint16_t);

int main(void)
{
	char data_buf[32];
	char paket_len = 24;
  unsigned int status = 0;
	
  uart_init();

	uart_send_text_p(Text_Reset);

	RFM01_init();
	
	uart_send_text_p(Text_Init);

	status = RFM01_read_status();

	show_status(status);


	while (1)
	{	 
	  RFM01_read_fifo(data_buf, paket_len);	

		status = RFM01_read_status();
		show_status(status);

    if ((data_buf[0] & 0xF0) == 0x50)         // neuer Sensor
      {
/*
        for (int i=0; i < paket_len; i++)
          {
            uart_send_byte_hex(data_buf[i]);
            uart_send_byte(' ');
          }
        uart_send_text("\r\n");
*/
    	  if (check_paket(data_buf, paket_len))
	        {
            show_sensor(data_buf);
	        }

      }
  }
}

//------------------------------------------------------------------

char check_paket(char *data_buf, char len)
{
  unsigned char i, check=0, flag=0;

	  for (i=0; i<len; i++)
	    {
          check = check ^ data_buf[i];	   
		  }
      if (!(check))                             // 0 wenn ok
  	    {
	    	  flag = 1;
		    }  
  return flag;
}

//------------------------------------------------------------------

void show_sensor(char *data_buf)
{
  uint8_t i, typ;

  uart_send_byte('S');
  uart_send_byte((data_buf[0] & 0x0F) | 0x30);			// Sensornummer 
  uart_send_byte('-');
  uart_send_byte((data_buf[1] & 0x0F) | 0x30);			// Sensortstatus
  uart_send_byte(':');


  for (i=2; i<21; i+=7)
    {
      typ = data_buf[i] & 0x0F;
      switch (typ)
        {
          case 1:
            show_temp(data_buf, i);
            break;

          case 2:
            show_feuchte(data_buf, i);
            break;

          case 3:
            show_druck(data_buf, i);
            break;
                                              
          case 4:
            show_switch(data_buf, i);
            break;

        }    
	  }

  uart_send_text("\r\n");

}

//------------------------------------------------------------------

void show_temp(char *data_buf, uint8_t pos)
{

  uart_send_byte(' ');	

  if (data_buf[pos+1] == '-')
    {
		  uart_send_byte(data_buf[pos+1]);   
		}
  else
    {
		  uart_send_byte(' ');   
		}

  if (data_buf[pos+2] == '0')
    {
		  uart_send_byte(' ');   
		}
  else
    {
		  uart_send_byte(data_buf[pos+2]);   
		}

  uart_send_byte(data_buf[pos+3]);		// Einer Grad
  uart_send_byte(data_buf[pos+4]);		// Komma
	uart_send_byte(data_buf[pos+5]);		// Zehntel Grad
  uart_send_text_p(Text_Grad);		
}

//------------------------------------------------------------------

void show_feuchte(char *data_buf, uint8_t pos)
{

  uart_send_byte(' ');	

  uart_send_byte(data_buf[pos+2]);	// Zehner Feuchte
  uart_send_byte(data_buf[pos+3]);	// Einer Feuchte
  uart_send_text_p(Text_Feuchte);		
}

//------------------------------------------------------------------

void show_druck(char *data_buf, uint8_t pos)
{

  uart_send_byte(' ');	

  if (data_buf[pos+1] == '0')
    {
		  uart_send_byte(' ');   
		}
  else
    {
		  uart_send_byte(data_buf[pos+1]);   
		}
  uart_send_byte(data_buf[pos+2]);	// Hunderter Luftdruck
  uart_send_byte(data_buf[pos+3]);	// Zehner Luftdruck
  uart_send_byte(data_buf[pos+4]);	// Einer Luftdruck
  uart_send_text_p(Text_Luftdruck);		
}

//------------------------------------------------------------------

void show_switch(char *data_buf, uint8_t pos)
{

  uart_send_byte(' ');	

  if ((data_buf[pos+1] == '0') && (data_buf[pos+1] == '0'))
    {
		  uart_send_text("Aus");   
		}
  else if ((data_buf[pos+1] == '9') && (data_buf[pos+1] == '9'))
    {
		  uart_send_text("Ein");   
		}
  else
    {
		  uart_send_text("Fehler");   
		}
}

void show_status(unsigned int status)
{
	uart_send_text_p(Text_Status);

	for (char i=0; i<16; i++)
		{
			if (status & 0x8000)
				{
          uart_send_byte('1');
        }
			else
				{
					uart_send_byte('0');
				}	  
			for (char j=0; j<2; j++)
				{
					uart_send_byte(' ');
				}
			status = (status<<1);					
		}
  uart_send_byte('\r');
  uart_send_byte('\n');
}

